searchData={"items":[{"type":"module","title":"Envio","doc":"Main interface to Envio.\n\nProvides handy functions to publish messages, subscribe to messages, etc.","ref":"Envio.html"},{"type":"function","title":"Envio.register/2","doc":"","ref":"Envio.html#register/2"},{"type":"behaviour","title":"Envio.Backend","doc":"The behaviour to be implemented for all the backends.","ref":"Envio.Backend.html"},{"type":"callback","title":"Envio.Backend.on_envio/2","doc":"The callback when the envio is received.","ref":"Envio.Backend.html#c:on_envio/2"},{"type":"module","title":"Envio.Channels","doc":"Channels storage.\n\nIt manages all the channels currently existing in the system.","ref":"Envio.Channels.html"},{"type":"function","title":"Envio.Channels.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Envio.Channels.html#child_spec/1"},{"type":"function","title":"Envio.Channels.init/1","doc":"Default initialization callback (noop.)","ref":"Envio.Channels.html#init/1"},{"type":"function","title":"Envio.Channels.register/2","doc":"Registers new channel.","ref":"Envio.Channels.html#register/2"},{"type":"function","title":"Envio.Channels.start_link/1","doc":"Starts a new channels bucket.","ref":"Envio.Channels.html#start_link/1"},{"type":"function","title":"Envio.Channels.state/0","doc":"Get list of active subscriptions.","ref":"Envio.Channels.html#state/0"},{"type":"module","title":"Envio.Utils","doc":"Multipurpose utils, required for the _Envío_ proper functioning, like:\n\n* `fq_name` to construct fully-qualified names for channels to prevent clashes","ref":"Envio.Utils.html"},{"type":"function","title":"Envio.Utils.config_value/1","doc":"Returns the anonymous function that either loads the system environment\nvariable or simply returns the value statically loaded from config file.","ref":"Envio.Utils.html#config_value/1"},{"type":"function","title":"Examples - Envio.Utils.config_value/1","doc":"iex> # config :envio, :binary_value, \"FOO\"\n    iex> Envio.Utils.config_value(Application.get_env(:envio, :binary_value)).()\n    \"FOO\"\n\n    iex> # config :envio, :env_value, {:system, \"FOO\"}\n    iex> System.put_env(\"FOO\", \"42\")\n    iex> Envio.Utils.config_value(Application.get_env(:envio, :env_value)).()\n    \"42\"","ref":"Envio.Utils.html#config_value/1-examples"},{"type":"function","title":"Envio.Utils.fq_name/2","doc":"Produces a fully-qualified name out of namespace in a form\n  of a module name atom and a method name (atom or binary.)","ref":"Envio.Utils.html#fq_name/2"},{"type":"function","title":"Examples - Envio.Utils.fq_name/2","doc":"iex> Envio.Utils.fq_name(\"foo\")\n    \"foo\"\n    iex> Envio.Utils.fq_name({Foo.Bar, \"baz\"})\n    \"foo/bar.baz\"\n    iex> Envio.Utils.fq_name(Foo.Bar, \"baz\")\n    \"foo/bar.baz\"\n    iex> Envio.Utils.fq_name(Foo.Bar, :baz)\n    \"foo/bar.baz\"\n    iex> Envio.Utils.fq_name(\"Foo.Bar\", \"baz\")\n    \"Foo.Bar.baz\"","ref":"Envio.Utils.html#fq_name/2-examples"},{"type":"function","title":"Envio.Utils.get_delete/3","doc":"Fetches the value from the map by given key _and_ returns both the value and\nthe map without this key.","ref":"Envio.Utils.html#get_delete/3"},{"type":"function","title":"Examples - Envio.Utils.get_delete/3","doc":"iex> Envio.Utils.get_delete(%{foo: :bar, baz: 42}, :foo)\n    {:bar, %{baz: 42}}\n    iex> Envio.Utils.get_delete(%{baz: 42}, :foo)\n    {nil, %{baz: 42}}\n    iex> Envio.Utils.get_delete(%{baz: 42}, :foo, :bar)\n    {:bar, %{baz: 42}}","ref":"Envio.Utils.html#get_delete/3-examples"},{"type":"function","title":"Envio.Utils.smart_to_binary/1","doc":"Safely converts any term to binary.","ref":"Envio.Utils.html#smart_to_binary/1"},{"type":"function","title":"Examples - Envio.Utils.smart_to_binary/1","doc":"iex> Envio.Utils.smart_to_binary(\"foo\")\n    \"foo\"\n    iex> Envio.Utils.smart_to_binary(42)\n    \"42\"\n    iex> Envio.Utils.smart_to_binary(%{foo: :bar, baz: 42})\n    \"%{foo: :bar, baz: 42}\"\n    iex> Envio.Utils.smart_to_binary([foo: :bar, baz: 42])\n    \"[foo: :bar, baz: 42]\"","ref":"Envio.Utils.html#smart_to_binary/1-examples"},{"type":"behaviour","title":"Envio.Publisher","doc":"Publisher helper scaffold.\n\nSimply `use Envio.Publisher` in the module that should publish messages.\nThe `broadcast/2` function becomes available. If the optional `channel:`\nargument is passed to `use Envio.Publisher`, this channel is considered\nthe default one and `broadcast/1` function appears to publish directly\nto the default channel.\n\nThe ready-to-copy-paste example of usage would be:\n\n```elixir\ndefmodule MyPub do\n  use Envio.Publisher, channel: :main\n\n  def publish(channel, what), do: broadcast(channel, what)\n  def publish(what), do: broadcast(what)\nend\n```\n\nAll the _subscribers_ of the particular channel the message was published\nto, will either receive a message (in the case of\n[`:pub_sub`](https://hexdocs.pm/elixir/master/Registry.html#module-using-as-a-pubsub))\nor called back with the function provided on subscription\n([`:dispatch`](https://hexdocs.pm/elixir/master/Registry.html#module-using-as-a-dispatcher)).\n\nSince `v0.8.0` `Envío` supports [`Phoenix.PubSub`](https://hexdocs.pm/phoenix_pubsub) for distributed message broadcasting.\n\nThe publisher does not wrap [`:via`](https://hexdocs.pm/elixir/master/Registry.html#module-using-in-via)\nfunctionality since it makes not much sense.\n\nFor how to subscribe, see `Envio.Subscriber`.","ref":"Envio.Publisher.html"},{"type":"callback","title":"Envio.Publisher.broadcast/2","doc":"The callback to publish stuff to `Envio`.","ref":"Envio.Publisher.html#c:broadcast/2"},{"type":"behaviour","title":"Envio.Subscriber","doc":"Subscriber helper scaffold.\n\nTo easy register the `pub_sub` consumer in `Envio.Registry`, one might\nuse this helper to scaffold the registering/unregistering code.\nIt turns the module into the `GenServer` and provides the handy wrapper\nfor the respective `handle_info/2`. One might override\n`handle_envio` to implement custom handling.\n\nThe typical usage would be:\n\n```elixir\ndefmodule PubSubscriber do\n  use Envio.Subscriber, channels: [{PubPublisher, :foo}]\n\n  def handle_envio(message, state) do\n    with {:noreply, state} <- super(message, state) do\n      IO.inspect({message, state}, label: \"Received message\")\n      {:noreply, state}\n    end\n  end\nend\n```\n\nIf channels are not specified as a parameter in call to `use Envio.Subscriber`,\nthis module might subscribe to any publisher later with `subscribe/1`:\n\n```elixir\nPubSubscriber.subscribe(%Envio.Channel{source: PubPublisher, name: :foo})\n```\n\nFor how to publish, see `Envio.Publisher`.","ref":"Envio.Subscriber.html"},{"type":"callback","title":"Envio.Subscriber.handle_envio/2","doc":"The callback to subscribe stuff to `Envio`.","ref":"Envio.Subscriber.html#c:handle_envio/2"},{"type":"module","title":"Envio.Channel","doc":"Channel description.","ref":"Envio.Channel.html"},{"type":"function","title":"Envio.Channel.fq_name/1","doc":"","ref":"Envio.Channel.html#fq_name/1"},{"type":"type","title":"Envio.Channel.t/0","doc":"Channel data stored as a struct","ref":"Envio.Channel.html#t:t/0"},{"type":"module","title":"Envio.State","doc":"Global Envio state. Contains subscriptions, messages\nthat were not yet processed and options.","ref":"Envio.State.html"},{"type":"type","title":"Envio.State.t/0","doc":"Internal state of everything amongst Envío.","ref":"Envio.State.html#t:t/0"},{"type":"exception","title":"Envio.InconsistentUsing","doc":"An exception to be thrown when an attempt to use scaffolding is inconsistent.\n\nFor instance, whether the call to `use Envio.Subscriber/1` has no valid `GenServer` defined.","ref":"Envio.InconsistentUsing.html"},{"type":"extras","title":"Envío","doc":"# ![Envío Logo](https://github.com/am-kantox/envio/blob/master/stuff/logo-48x48.png?raw=true)   Envío\n\n**Envío** is basically a `GenEvent²`, the modern idiomatic pub-sub\nimplementation of event passing.\n\nIn a nutshell, **Envío** is a set of handy tools to simplify dealing with Elixir\n[`Registry`](https://hexdocs.pm/elixir/master/Registry.html). It includes\nthe instance of `Registry` to be used out of the box, scaffolds for\nproducing _publishers_ and _subscribers_.\n\nIt is built using “convention over configuration” approach, preserving the whole\nlow-level control over the registry entries.\n\nJust add the application `:envio` into your list of _extra_ applications\nand the default `Envio.Registry` will be started and managed automagically.\n\n```elixir\ndef application do\n  [\n    mod: {MyApplication, []},\n    extra_applications: ~w|envio ...|a\n  ]\nend\n```\n\n#","ref":"envio.html"},{"type":"extras","title":"Creating a publisher - Envío","doc":"To create a publisher just `use Envio.Publisher` in the module that should\npublish messages to the subscribers. Once `use Envio.Publisher` is used,\n`broadcast/2` function becomes available. If the optional `channel:`\nargument is passed to `use Envio.Publisher`, this channel is considered\nthe default one and `broadcast/1` function appears to publish directly\nto the default channel.\n\n```elixir\ndefmodule MyPub do\n  use Envio.Publisher, channel: :main\n\n  def publish(channel, what), do: broadcast(channel, what)\n  def publish(what), do: broadcast(what)\nend\n```\n\nAnother option that might be passed to `use Envio.Publisher` is `manager:`, that might be either `:registry` (default,) or `:phoenix_pub_sub` to use [`Phoenix.PubSub`](https://hexdocs.pm/phoenix_pubsub) for distributed message broadcasting.\n\n#","ref":"envio.html#creating-a-publisher"},{"type":"extras","title":"Creating a subscriber - Envío","doc":"##","ref":"envio.html#creating-a-subscriber"},{"type":"extras","title":"▶ [`:dispatch`](https://hexdocs.pm/elixir/master/Registry.html#module-using-as-a-dispatcher) - Envío","doc":"Simply register the handler anywhere in the code:\n\n```elixir\nEnvio.register(\n  {MySub, :on_envio}, # the function of arity 2 must exist\n  dispatch: %Envio.Channel{source: MyPub, name: :main}\n)\n```\n\nAs `MyPub` publishes to the `:main` channel, `MySub.on_envio/2` will\nbe called with a message passed as parameter.\n\n##","ref":"envio.html#dispatch-https-hexdocs-pm-elixir-master-registry-html-module-using-as-a-dispatcher"},{"type":"extras","title":"▶ [`:pub_sub`](https://hexdocs.pm/elixir/master/Registry.html#module-using-as-a-pubsub) - Envío","doc":"Use `Envio.Subscriber` helper to scaffold the registry subscriber. Implement\n`handle_envio/2` for custom message handling. The default implementation\ncollects last `10` messages in it’s state. This amount might be adjusted by\nchanging `:envio, :subscriber_queue_size` application environment setting.\n\nThe implementation below subscribes to `:main` channel provided by `MyPub`\npublisher and prints out each subsequent incoming message to standard output.\n\n```elixir\ndefmodule PubSucker do\n  use Envio.Subscriber, channels: [{MyPub, :main}]\n\n  def handle_envio(message, state) do\n    {:noreply, state} = super(message, state)\n    IO.inspect({message, state}, label: \"Received\")\n    {:noreply, state}\n  end\nend\n```\n\n##","ref":"envio.html#pub_sub-https-hexdocs-pm-elixir-master-registry-html-module-using-as-a-pubsub"},{"type":"extras","title":"▶ [`:phoenix_pub_sub`](https://hexdocs.pm/phoenix_pubsub) - Envío","doc":"Use `manager: :phoenix_pub_sub` for distributed message broadcasting. The implementation below subscribes to `\"main\"` channel in the distributed OTP environment and prints out each subsequent incoming message to standard output.\n\n```elixir\ndefmodule Pg2Sucker do\n  use Envio.Subscriber, channels: [\"main\"], manager: :phoenix_pub_sub\n\n  def handle_envio(message, state) do\n    {:noreply, state} = super(message, state)\n    IO.inspect({message, state}, label: \"Received\")\n    {:noreply, state}\n  end\nend\n```\n\nThe publisher this subscriber might be listening to would look like\n\n```elixir\ndefmodule Pg2Spitter do\n  use Envio.Publisher, manager: :phoenix_pub_sub, channel: \"main\"\n  def spit(channel, what), do: broadcast(channel, what)\n  def spit(what), do: broadcast(what)\nend\n```","ref":"envio.html#phoenix_pub_sub-https-hexdocs-pm-phoenix_pubsub"},{"type":"extras","title":"Backends","doc":"# Backends\n\n**Envío** supports configurable backends as subscribers with very little to none\ncode. It comes with the following backends out of the box:\n\n* `Slack` — a subscriber to _envíos_ that sends all the incoming messages to the preconfigured slack channel\n* `IO`, `...`, _more coming_\n\nBackend is basically a supervised `GenServer` that implements `Envio.Backend`\nbehaviour. Backends, provided by the core are generated based on the configuration.\nThe typical config looks like this (example for `Envio.Slack`):\n\n```elixir\nconfig :envio, :backends, %{\n  Envio.Slack => %{\n    {MyPub, :main} => [\n      hook_url: {:system, \"SLACK_ENVIO_HOOK_URL\"}\n    ]\n  }\n}\n```\n`Envio.Slack`, used as a key, is a name of the module that implements the\n`Envio.Backend` behaviour (currently it’s the single function\n[`Envio.Backend.on_envio/2`](Envio.Backend.html#c:on_envio/2)\nthat receives a message when published by the publisher.) The value assigned\nto the key is a map of `{Envio.Publisher, :channel}` tuples to the list of\narguments that will be injected into the message under `:meta` key.\n\nWhen the publisher emits, say,\n`%{title: \"Pi reminder\", text: \"Recall pi value now!\", pi: 3.14}` _envío_,\nthe `%{meta: %{hook_url: \"my_slack_hook_url\"}}` will be merged into the\nmessage for the further processing by the backend. Typically, the backend\nwill extract this `meta` information out of the message using\n\n```elixir\n{meta, message_without_meta} = Utils.get_delete(message, :meta)\n```\n\nand use this information for the backend-specific actions (e. g. what channel\nto post the message to in the case of Slack.)\n\n#","ref":"backends.html"},{"type":"extras","title":"Implementation details - Backends","doc":"`Slack` backends understands four _predefined_ keys:\n\n* `title` — is treated as title of the message\n* `text`, `message` — both are treated as a message body, formatted as `pretext`\n* `level` — the message “level” that affects the color of the left bar and the icon.\n\nAll other keys are treated as _values_, put as _short_ and _long_ attachments\ndepending on their length (those longer than 32 symbols take the whole line in\nthe slack attachments output.)\n\n#","ref":"backends.html#implementation-details"},{"type":"extras","title":"Slacking using `Envío` - Backends","doc":"It has never easier. Once your application uses `Envío` as a publisher,\nbroadcasting messages to the channel named `my_app/my_module.main_channel`\n(the latter means messages are published by `MyApp.MyModule` that has\n`use Envio.Publisher, channel: :main_channel`, using `broadcast(message)`,\nor you have a generic `pub_sub` publisher registered in `Envio.Registry`\nusing standard Elixir `Registry` functions,) simply add the following to\nthe configuration file:\n\n```elixir\nconfig :envio, :backends, %{\n  Envio.Slack => %{\n    {MyApp.MyModule, :main_channel} => [\n      # one might simply put a channel name here as binary,\n      #  but I don’t recommend that since it’s kinda credentials\n      hook_url: {:system, \"SLACK_ENVIO_HOOK_URL\"}\n    ]\n  }\n}\n```\n\nWell, that’s it. Your application is now Slack-enabled.\n\n#","ref":"backends.html#slacking-using-envi-o"},{"type":"extras","title":"Example from `Envío` tests - Backends","doc":"To test Envío’s backend functionality I used plain old good `IO.inspect`\nbackend with `ExUnit.CaptureIO` as a checker:\n\n```elixir\ndefmodule Envio.IOBackend do\n  @moduledoc false\n\n  @behaviour Envio.Backend\n\n  @impl Envio.Backend\n  def on_envio(message, meta) do\n    IO.inspect(message, label: \"[★Envío★]\")\n  end\nend\n```\n\nThe above is a fully valid backend implementation, that subscribes to the `:backends` channel\nof the `Spitter.Registry` ckass defined in `test_helper.exs` (the excerpt is from `test.exs` config):\n\n```elixir\nconfig :envio, :backends, %{\n  Envio.IOBackend => %{{Spitter.Registry, :backends} => []}\n}\n```\nOnce emitted by `Spitter.Registry`, this message will be published to the standard output:\n\n```elixir\n[★Envío★]: %{bar: 42, meta: %{}}\n```\n\nFor distributed backend over `Phoenix.PubSub.PG2` one might use the following config:\n\n```elixir\nconfig :envio, :backends, %{\n  Envio.IOBackend.PG2 => %{\"main\" => [manager: :phoenix_pub_sub]}\n}\n```","ref":"backends.html#example-from-envi-o-tests"}],"content_type":"text/markdown"}